class DependencyNotFoundException implements Exception {
  final Type dependencyType;

  DependencyNotFoundException(this.dependencyType);

  @override
  String toString() => 'DependencyNotFoundException: undeclared is $dependencyType';
}

class DependenciesContainer {
  final _cache = <Type, Object>{};
  final _factory = <Type, Object Function()>{};

  void update<T extends Object>(T Function(DependenciesContainer container) getter) {
    _factory[T] = () => getter(this);
  }

  U dependency<U>({bool existingInstance = true}) {
    if (existingInstance) {
      if (_cache[U] == null) {
        _cache[U] = _factory[U]!();
      }
      return _cache[U]! as U;
    } else {
      return _factory[U]!() as U;
    }
  }

  void clearSingleton<U>() => _cache.remove(U);

  void clear() {
    _cache.clear();
    _factory.clear();
  }
}
